﻿using Hotline.Share.Dtos.ExportWord;
using Novacode;

namespace Hotline.Application.ExportWord
{
    public class WordExportProvider : IWordExportProvider
    {
        public ExportByteWord ExportFromTemplate<T>(string templatePath, T data) where T : IWordExportTemplate
        {
            DocX docX = DocXHelper.GetDocX(templatePath);
            ReplacePlaceholders(docX, data);
            return new ExportByteWord
            {
                WordBytes = docX.ToBytes()
            };
        }

        public async Task<ExportByteWord> ExportFromTemplateAsync<T>(string templatePath, T data) where T : IWordExportTemplate
        {
            return await Task.Run(() => ExportFromTemplate(templatePath, data));
        }

        private void ReplacePlaceholders<T>(DocX word, T wordData) where T : IWordExportTemplate
        {
            if (word == null)
            {
                throw new ArgumentNullException("word");
            }

            IEnumerable<PlaceholderEntity> replacements = wordData.GetReplacements();
            if (replacements.Count() == 0)
            {
                throw new Exception("实体中没有可替换的属性");
            }

            DocXHelper.ReplacePlaceholdersInWord(word, replacements);
        }
    }
}
